
class Recipe:
    def __init__(self):
        self.name = "Unknown"
        self.author = "Unknown"
        self.description = ""
        self.yeild = 4
        self.category = "Unknown"
        self.ingredients = []
        self.instructions = []
        self.prep_time = "PT0M"
        self.cook_time = "PT0M"
        self.image = ""

    



# A scraper that, given an url, can parse the page and detect the ingredients
# and instructions in a page. It return the ingredients and instructions in a
# list and the informations about the recipe in a dictionnary (?)
#
# Easy case :
# To find the data we must first scrape the page for a script tag with the type
# "application/ld+json". This is a tag that is used by google to parse the data
# of recipes. If a recipe website wants to show their recipe in an enhance
# result, they must use this, therefore about 75% of cooking website use this.
#
# Hard case :
# The other 25% will be parsed in a different way where the arbitrary HTML will
# be analyzed. The technique is described more in depth in recipe_complex.py

import requests
import json
from bs4 import BeautifulSoup
from recipeGetter.recipe_trivial import RecipeTrivial
from recipeGetter.recipe_complex import RecipeComplex
from recipeGetter.recipe import Recipe

class RecipeScraper:
    def __init__(self, url):
        self.url = url
        self.content = None
        self.html = None
        self.error = None
        self.trivial_content = None
        self.recipe = None

    def scrape(self):
        self._scrape_content()
        if self.error == None:
            self.html = BeautifulSoup(self.content, 'html.parser')
            # If the case is not trivial, this function will automatically try
            # to find information based on the arbitrary HTML and return a
            # "RecipeComplex" instead of a "RecipeTrivial" instance
            self.recipe = self._find_trivial_case()
            return self.recipe
        return Recipe()
        #return self.error

    # This function will scrape the website specified by the url given when the
    # class was instanciated and put the content of the response in
    # "self.content". If an error occur, bubble up the error
    def _scrape_content(self):
        try:
            response = requests.get(self.url)
            response.raise_for_status()
            self.content = response.content
        except requests.exceptions.RequestException as err:
            self.error = err

    # This function will first find if the current website is using putting
    # it's recipe in the metadata (in a script tag with the type
    # application/ld+json). If the tag is found, the data will be parsed, if
    # the tag is not found we will have some more work to do.
    def _find_trivial_case(self):
        scripts_tag = self.html.find_all("script", attrs={"type":"application/ld+json"})
        if len(scripts_tag) > 0:
            for script_tag in scripts_tag:
                data = json.loads(script_tag.string)
                # Every recipe that is supported must have a "@type":"Recipe"
                # at the root of the JSON tree
                if "@type" in data.keys() and data["@type"] == "Recipe":
                    self.trivial_content = data
                    break
        if self.trivial_content == None:
            return self._parse_recipe()
        else:
            return self._parse_trivial_case()

           
    def _parse_trivial_case(self):
        return RecipeTrivial().parse(self.trivial_content)

    # This function is called only if the recipe is not in metadata and is
    # therefore not a trivial case. This function has more chance to fail than
    # a trivial case because we try to find the information about the recipe in
    # the complete html page.
    def _parse_recipe(self):
        return RecipeComplex(self.html).parse()

       

recette = RecipeScraper("https://www.ricardocuisine.com/recettes/5335-sauce-a-spaghetti-la-meilleure")
infos = recette.scrape()
print(infos.ingredients)
print(infos.instructions)



import isodate
from recipe import Recipe

class RecipeTrivial(Recipe):
    # This function will find the informtion needed (like ingredients and
    # instructions) in the json data.
    def parse(self, data):
        # self.recipe_ingredients = []
        # self.recipe_instructions = []

        self.name = data.get("name", "unknown")
        self.author = self._parse_author(data.get("author"))
        self.category = data.get("recipeCategory", "unknown")
        self.description = data.get("description", "")
        self.yeild = data.get("recipeYeild", 0)
        self.prep_time = self._parse_duration(data.get("prepTime", "PT0M"))
        self.cook_time = self._parse_duration(data.get("cookTime", "PT0M"))
        self.image = self._parse_images(data.get("image"))
        self.instructions = self._parse_instructions(data.get("recipeInstructions"))
        self.ingredients = data.get("recipeIngredient", [])
        return self

    # This function will get the author name depending if it is a person or an
    # organization.
    @staticmethod
    def _parse_author(author):
        if author.get("@type") == "Person":
            return author.get("name")
        elif author.get("@type") == "Organization":
            return author.get("legalName")
        else:
            return None

    # This function will parse the duration from the ISO 8601 format to a
    # number of minutes. parse_duration return a timedelta which the number of
    # seconds is taken and divided by 60 to have a number of minutes
    @staticmethod
    def _parse_duration(duration):
        isodate.parse_duration(duration).seconds // 60

    # The function will check if multiple image's urls are given (in a list) or
    # only one image url is given (directly a str)
    @staticmethod
    def _parse_images(images):
        if isinstance(images, list):
            return images[0]
        else:
            return images

    # This function will take a list of steps or sections and will transform it
    # into a list of section(s) with steps inside them. Each section is an
    # element in the sections list. A section is a dictionnary with 2 keys: a
    # name and a list of steps. If there is no name for a section, the section
    # name will be empty ("").
    @staticmethod
    def _parse_instructions(steps_sections):
        sections = []
        global_section_index = None
        for step_section in steps_sections:
            if step_section.get("@type") == "HowToStep":
                if global_section_index == None:
                    global_section_index = len(sections)
                    sections.append({"steps":[], "name":""})
                sections[global_section_index]["steps"].append(step_section.get("text", ""))

            elif step_section.get("@type") == "HowToSection":
                steps = []
                for step in step_section.get("itemListElement"):
                    steps.append(step.get("text", ""))
                sections.append({"steps":steps, "name":step_section.get("name", "unknown")})

        return sections




from recipe import Recipe

class RecipeComplex(Recipe):
    ingredient_titles = ["ingredients", "ingredient", "ingrédient",
                         "ingrédients", "you will need", 
                         "vous aurez besoins"]
    instruction_titles = ["steps", "directions", "instructions",
                          "step", "direction", "instruction",
                          "étape", "étapes", "préparation",
                          "prép", "prep", "method", "méthode"]
    # TODO: Hints should be splited by language. The language can be found in the meta tag of the page.
    ingredient_hints = ["g", "kg", "tbs", "cups", "cup",
                        "tasses", "tasse", "c. à soupe",
                        "teaspoon", "tablespoon", "cuillère",
                        "1/2", "1/4", "oz", "oil", "huile",
                        "butter", "beurre"]
    instruction_hints = ["mix", "mélanger", "cook", "cuire",
                         "feu", "heat", "moyen-élevé",
                         "medium-high", "season",
                         "ingredients", "ingrédients", "saler"]

    def __init__(self, html):
        self._html = html
        self._section_tag_type = ["h1", "h2", "h3", "h4", "h5", "h6"]
        super().__init__()

    # The class will try to find the recipe informations in arbitrary HTML. For
    # every information we can make a few assumptions. This is not a perfect
    # method but can give good result. Not all data is supported (like prep and
    # cooking time). Also, page in other language than English or French will
    # not give good results.
    def parse(self):
        self.name = self._find_name()
        self.author = self._find_author()
        self.description = self._find_description()
        self.ingredients = self._find_ingredients()
        self.instructions = self._find_instructions()
        return self

    # The function will try to find the name of the recipe. It will first try
    # to find the og:title meta tag. If the tag was not found it will take the
    # title tag as the recipe is generally put in the title of the website.
    def _find_name(self):
        og_title = self._html.find("meta", attrs={"property", "og:title"})
        if not og_title == None:
            return og_title.attrs.get("content", "")
        else:
            return self._html.find("title").string

    def _find_description(self):
        description = self._html.find("meta", attrs={"name", "description"})
        if not description == None:
            return description.attrs.get("content", "")

    def _find_author(self):
        author = self._html.find("meta", attrs={"name", "author"})
        if not author == None:
            return author.attrs.get("content", "")

    # This function will try to find the section where ingredients are placed.
    # If found it will try to get all the ingredients
    def _find_ingredients(self):
        return self._find_section(self.ingredient_titles, self.ingredient_hints, 0, 75)

        
    def _find_instructions(self):
        instructions = self._find_section(self.instruction_titles, self.instruction_hints, 100, 500)
        return [{
            "name":"",
            "instructions":instructions
        }]
    # Every h1 to h6 will be search for a word like ingredient (or
    # instructions) (and variations in differrent languages). If the header
    # (h..) is found, return it. If a header was previously found, (for
    # ingredients) use the same type of header to search for instructions.
    def _find_section_title(self, keywords):
        for tag_type in self._section_tag_type:
            for header in self._html.find_all(tag_type):
                for keyword in keywords:
                    if not header.string == None and header.string.lower() == keyword:
                        self.section_tag_type = [header.name]
                        return header
        return None


    # This function will try to find keywords that are related to ingredients
    # (ml, g, cups, tasse, tbs, etc..) in the child of the given parent.
    # Every time a keyword is found, the immediat parent of the keyword is put
    # inside a list (it's tag and class). The most common tag and class with a
    # keyword will be selected and, with that, all tag with that class in a
    # same level will become ingredients.
    @staticmethod
    def _find_section_list(parent, hints, min_len, max_len):
        matches = []
        for tag in parent.find_all():
            text = tag.string
            if not text == None and len(text) < max_len and len(text) > min_len:
                for word in text.split(" "):
                    for hint in hints:
                        if hint == word:
                            if "class" in tag:
                                matches.append(f"{tag.name}:{tag['class'][0]}")
                            else:
                                # Not the best..
                                matches.append(f"{tag.name}")
        if len(matches) > 0:
            most_frequent = max(set(matches), key = matches.count).split(":")
            if len(most_frequent) == 2:
                return parent.find_all(most_frequent[0], attrs={"class":most_frequent[1]})
            elif len(most_frequent) > 0:
                return parent.find_all(most_frequent[0])
        else:
            return None


    # This function will try to find the list of element in the section
    # (ingredient or instructions) based on the position of the title of the
    # section. If the title of the section is not found, the function will
    # return nothing. If the section list is not found based on the parent of
    # the title, the next parent will be taken, if not found again, the
    # function will stop and nothing will be returned
    def _find_section(self, titles, hints, min_len, max_len):
        section_title = self._find_section_title(titles)
        if not section_title == None:
            section_list = self._find_section_list(section_title.parent, hints, min_len, max_len)
            if section_list == None:
                section_list = self._find_section_list(section_title.parent.parent, hints, min_len, max_len)
            if not section_list == None:
                section_list = list(map(lambda ingredient: ingredient.string,section_list))
                return section_list
        return []



from distutils.core import setup
setup(
  name = 'recipeGetter',
  packages = ['recipeGetter'],
  version = '0.0.2',
  license='MIT',
  description = 'A scraper for recipe website',  
  author = 'Xavier Hamel',
  author_email = 'xavier.hamel.protic@gmail.com',
  url = 'https://github.com/xavierhamel/recipeGetter',
  download_url = 'https://github.com/xavierhamel/recipeGetter/archive/v_02.tar.gz',
  keywords = ['recipe', 'scraper', 'getter'],
  install_requires=[      
          'requests',
          'beautifulsoup4',
      ],
  classifiers=[
    'Development Status :: 4 - Beta',      
    'Intended Audience :: Developers',     
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',    
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.9',
  ],
)
